* ToText.PRG

/*
  By Roger Seiler - first written in 12/96; many revisions since then.

  This is a tool for converting old Clipper DOS reports into Windows
  reports that are printed via TRept23 and TReport.

  ( Also includes MemWrap() function for selecting one word-wrapped line 
  at a time for printing. This and GetText() and GetText2() are called
  from outside - not called from proc ToText(). )

  The main concept here is that a report is constructed inside a cText 
  filewide static, line by line. Each new line of text is constructed
  field by field into cNewLine.  When finished building a line in cNewLine,
  then cNewLine is added to the end of cText, and then the next new line
  is started.  When the report is finished being assembled, then the calling 
  proc makes a call to the GetText() function here to retrieve cText and then 
  sends it on to TRept23() for printing.

  The text sent to this proc in "cLine" uses the standard Clipper 
  "@ nRow, nCol SAY text" format, so we use AT() to find the "@" symbol, 
  the "," and the "SAY" word in order to find nRow, nCol and the text that
  we need to work with here.  In addition, if the text element comes from a 
  memvar instead of being passed as a quoted string, i.e. "some text", then
  it gets passed as a parameter in place of the dummy cMemvar. But if a quoted
  string is used for the text in the "@ nRow, nCol SAY text" statement,
  then the empty dummy cMemvar is passed.

  Note that this proc currently assumes that cText will not build up to
  over 64K.  If it does, then whoops, you've got a GPF (Go Pray Faithfully).

*/

#include "FiveWin.ch"

STATIC cText,cNewLine,nRowA,nRowB

 // Print to text file...
PROCEDURE ToText(cLine,cMemvar)

// This function analyzes each @ SAY statement in order to position each piece
// of text correctly on the page, building the text one line at a time, and
// then adds each line to a string that will be written to a text file.

LOCAL nAt1,nAt2,nAt3,nCol,cBlanks
LOCAL nBlanks := 0

DEFAULT cText := ""
DEFAULT cNewLine := ""
DEFAULT nRowA := 0
DEFAULT nRowB := 0

nRowB := 0

 // Find row number:
nAt1 := AT("@", cLine)
nAt2 := AT(",", cLine)
  // Extract row # from "nRow" memvar in cLine. This works because nRow
  // is a global public memvar declared in calling prg, so it is visible to
  // the macro operator. Note that "nRow" itself is not even referenced 
  // as a memvar in this proc...
nRowB := &( ALLTRIM(SUBSTR(cLine,nAt1+1,(nAt2-1-nAt1))) )

 // If new page started...
IF nRowA > nRowB
   nRowA := 0
   cText := cText + cNewLine+CRLF
   cNewLine := ""
/*
      // Make sure if starting a new page, that prior page ends with a CRLF...
   IF LEN(cText) > 2
      IF RAT(CHR(13)+CHR(10),cText) # LEN(cText) - 2
         cText := cText + CRLF
      ENDIF
   ENDIF
*/
ENDIF

 // Do only at beginning of first pass of a page:
IF nRowA= 0
  DO WHILE nRowA# nRowB
   nRowA:= nRowA+ 1
   IF nRowB > nRowA
        cText := cText + CRLF
   ENDIF
   SysRefresh()
   CursorWait()
  ENDDO
ENDIF

// Find column number:
nAt3 := AT("SAY", cLine)
IF nAt3 = 0  // Try again if not found above - maybe upper & lower case:
   nAt3 := AT("Say", cLine)
ENDIF
IF nAt3 = 0  // If still not found, maybe all lower case:
   nAt3 := AT("say", cLine)
ENDIF
IF nAt3 = 0
   MsgAlert('Error in finding "SAY" in line ' + STR(nRowB) )
   RETURN 
ENDIF

nCol := VAL( SUBSTR(cLine,nAt2+1,(nAt3-1-nAt2)) )

// If moving to a new row, then add the line just finished to cText memvar,
// and tack on a carriage return at end of the line:
IF nRowB > nRowA .AND. nRowA > 0
     cText := cText + cNewLine+CRLF
  // Initialize a new line:
  cNewLine := ""
  // If next line to be printed is more than one line down the page, 
  // then add appropriate # of carriage returns to get to it:
  DO WHILE nRowA# nRowB
   nRowA:= nRowA+ 1
   IF nRowB > nRowA
      cText := cText + CRLF
   ENDIF
   SysRefresh()
   CursorWait()
  ENDDO
ENDIF

// If we need to space over to where next piece of text is to be added to a 
// line, then determine # of blank spaces needed and add them onto the line:
nBlanks := (nCol - 1) - LEN(cNewLine)
cBlanks := SPACE(nBlanks)
IF nBlanks > 0
   cNewLine := cNewLine + cBlanks 
ENDIF

// If this piece of text to be added was passed here in cMemvar, then add it
// to the line:
IF SUBSTR(cLine, nAt3 + 4, 1) # '"'  //!EMPTY(cMemvar)
     cNewLine := cNewLine + cMemvar
   ELSE // If text piece to be added is in a quote instead of cMemvar:
     cNewLine := cNewLine + SUBSTR(cLine, nAt3 + 5, LEN(cLine)-(nAt3+5))
ENDIF      

RETURN

*-----------------------------
  // Break text from a memo into word-wrapped single line increments...
FUNCTION MemWrap(cMem,nLnStart,nLnEnd,nTxtLen,nRow)
LOCAL cLine := "",nLen := 0,cSel := "",nEnd := 0,nChunk := 0,nSpace := 0
STATIC cMemo := ""
DEFAULT nLnStart := 1
DEFAULT nLnEnd := 70

IF !EMPTY(cMem)
   cMemo := cMem
      // Clean it up:
      // Get rid of soft CRLF's in text...
   cMemo := STRTRAN(cMemo, CHR(141) + CHR(10), "",,)
   cMemo := RTRIM(cMemo)
   DO WHILE .t.  // Get rid of excess line feeds at end of text.
      nLen := LEN(cMemo)
      IF SUBSTR(cMemo,nLen-1,2) = CHR(13)+CHR(10)
         cMemo := SUBSTR(cMemo,1,nLen-2)
        ELSE
         EXIT
      ENDIF        
      SysRefresh()
      CursorWait()
   ENDDO
ENDIF

IF !EMPTY(cMemo)
   nLen := LEN(cMemo)
      // nLnEnd is max allowable line length; nLenStart is how many spaces
      // in from left margin that we want to start printing...
   nChunk := nLnEnd - nLnStart + 2
      // Initially select line text as a portion equal to max line length
      // plus 1 char (in case last word ends exactly at max length and is
      // followed by a space or CHR(13) so we can include that word)...
   cSel := SUBSTR(cMemo,1,nChunk)
      // Select line text from portion of cMemo up to the first CR...
   nEnd := AT(CHR(13),cSel)
      // If no CR in selected line, then word wrap by cutting off end of line
      // back to the first space, but only if sel line is > max line len...
   IF nEnd = 0 .AND. LEN(cSel) = nChunk
      nEnd := RAT(" ",cSel)
   ENDIF
      // In case selected line consists of just one word...
   IF nEnd = 0
      nEnd := LEN(cSel)
     ELSE
      nEnd--  // Cut off CR or empty space at end
   ENDIF
      // Adjust selected line to contain only portion up to nEnd...
   cSel := SUBSTR(cSel,1,nEnd)
      // Cut cSel from beginning of cMemo to prepare cMemo for next loop...
   IF LEN(cMemo) > nEnd
      cMemo := SUBSTR(cMemo,nEnd+1,nLen)
      // Get rid of any CRLF at beginning of cMemo...
      IF SUBSTR(cMemo,1,2) = CHR(13)+CHR(10)
         IF LEN(cMemo) > 2
            cMemo := SUBSTR(cMemo,3,nLen)
           ELSE
            cMemo := ""
         ENDIF
      ENDIF
      // Get rid of any single blank space at beg of cMemo (left over from 
      // word wrapping)....
      IF SUBSTR(cMemo,1,1) = " "
         IF LEN(cMemo) > 1
            cMemo := SUBSTR(cMemo,2,nLen)
           ELSE
            cMemo := ""
         ENDIF
      ENDIF
     ELSE
      cMemo := ""
   ENDIF
ENDIF
nTxtLen := LEN(cMemo)
cLine := cLine + cSel

RETURN cLine


*----------------------------- End ToText()
// Enable static cText to be imported into another PRG. and reset cText.
FUNCTION GetText() 
LOCAL cTextB
IF EMPTY(cText)
   cText := ""
ENDIF
IF EMPTY(cNewLine)
   cNewLine := ""
ENDIF

  cText := cText + cNewLine+CRLF  // Make sure last line of text included.
  cTextB := cText
  cText := ""
  cNewLine := ""

RETURN cTextB  
*-----------------------------
// Enable static cText to be imported into another PRG. to check it.
FUNCTION GetText2() 
LOCAL cTextB
  cTextB := cText
RETURN cTextB  
*-----------------------------

/* Sample report code that calls ToText() to assemble the report as a
   text memvar and then saves that memvar to disk as a text file which
   will be read and printed by TRept23. Alternatively, the text memvar can
   just be passed directly to TRept23():New() as the second parameter leaving 
   the first parameter empty.

// A code excerpt from a typical report...

// Max Page length = 55
IF(nRow < 55,nRow++,nRow := 1)

   // Report header...
IF nRow = 1
   nPage++
   cMemvar := "" // <--- This dummy memvar should be made local

// Just take the old Clipper report code of the @...SAY type, for example:
//     @ 5, 10 SAY "Whatever"  
// and convert it into parameters of the ToText() function call, 
// as for example...

// ToText('@ 5, 10 SAY "Whatever" ', cMemvar)
// Note above the use of single quotes and the dummy parameter cMemvar.

// But where the SAY is a memvar or the result of some function, then replace
// the dummy cMemvar with the real memvar or functional expression, as shown
// below. Also below, note how font and printhead positions are inserted into
// the code for use later by TRept23...

   ToText('@ nRow,   1 SAY DTOC(DATE())', "<f1>"+DTOC(DATE()))
   ToText('@ nRow,  11 SAY TIME()', TIME())    // Passing result of a function.
   ToText('@ nRow,  26 SAY "<f1 p850>Responses Tabulation"', cMemvar)
   ToText('@ nRow,  54 SAY "<f1 p1560>SurCalc4   "', cMemvar)
   ToText('@ nRow,  64 SAY "Page No."', cMemvar)
   ToText('@ nRow,  73 SAY LTRIM(STR(nPage))', LTRIM(STR(nPage)))
   nRow++
   cSurvey := PADC(ALLTRIM(cSurvname),50)
   cSurvey := "<f2 p650>"+cSurvey
   ToText('@ nRow,  11 SAY cSurvey', cSurvey)  // Passing a real memvar here.
   nRow++
   ToText('@ nRow,   1 SAY REPLICATE("_",75)', "<f4>"+REPLICATE("_",75))
   nRow++
   nRow++
ENDIF

   // Print totals triggered by lL4...
IF nRow > 4
    // If start of question is past 55 lines, then push it to next page...
  IF (nRow + nAns + 1) > 55
     nRow := 1
     nPage++
     Header(nPage,cSurvName)
  ENDIF

  cLevel4 := STR(nLevel4,6,2)
  IF SUBSTR(cLevel4,6,1) == "0"
     cLevel4 := SUBSTR(cLevel4,1,5)+" "
     IF SUBSTR(cLevel4,5,1) == "0"
        cLevel4 := SUBSTR(cLevel4,1,3)+"   "
     ENDIF
  ENDIF
  cLevel4 := ALLTRIM(cLevel4)

  IF Itemtyp == "Q"
   ToText('@ nRow,   2 SAY "<f2>Q"', cMemvar)
   ToText('@ nRow,  4 SAY cLevel4+":"', cLevel4+":")
  ENDIF

     // Print the first line of the memo...
  cQ1 := MemWrap(cQtn,9,nLnEnd,@nTxtLen,nRow)  // MemWrap() in ToText.prg
  cQ1 := "<f1>"+cQ1
  ToText('@ nRow,  10 SAY Q1', cQ1)
  nRow++

     // Print all subsequent lines of the memo...
     // (nCnt is a safety factor to prevent endless loop)
  DO WHILE nTxtLen > 0 .AND. nCnt < 25
     cQ2 := MemWrap("",10,nLnEnd,@nTxtLen,nRow)
     cQ2 := "<f1>"+cQ2

     ToText('@ nRow,  10 SAY Q2', cQ2)
     IF(nRow < 55,nRow++,nRow := 1)
     nCnt++
       // Report header...
     IF nRow = 1
        nPage++
        Header(nPage,cSurvName)
     ENDIF
     SysRefresh()
     CursorWait()
  ENDDO
  nCnt := 0

  nRow++


// And so on through the body of the report.

// Then...

// save cText to disk
cText := GetText()  // Retreive cText from ToText.prg, where it is filew static
MEMOWRIT("MyReport.txt", cText)


oRept23 := TRept23():New("MyReport.txt","",,,.t.)  // Now print the report.
oRept23:Activate()
oRept23 := nil


*/